@@ -55,6 +55,20 @@ def get_tenant_field(model_class_or_instance):
                          model_class_or_instance, tenant_column))
 
 
+def get_object_tenant(instance):
+    field = get_tenant_field(instance)
+
+    if field.primary_key:
+        return instance
+
+    return getattr(instance, field.name, None)
+
+
+def set_object_tenant(instance, value):
+    if instance.tenant_value is None and value and not isinstance(value, list):
+        setattr(instance, instance.tenant_field, value)
+
+
 def get_current_tenant_value():
     current_tenant = get_current_tenant()
     if not current_tenant:
